package xyz.lychee.lagfixer.libs.adventure.text.format;

import java.util.EnumMap;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;
import xyz.lychee.lagfixer.libs.adventure.key.Key;
import xyz.lychee.lagfixer.libs.adventure.text.event.ClickEvent;
import xyz.lychee.lagfixer.libs.adventure.text.event.HoverEventSource;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.NonExtendable;

@NonExtendable
public interface StyleSetter<T extends StyleSetter<?>> {
   @NotNull T font(final @Nullable Key font);

   @NotNull T color(final @Nullable TextColor color);

   @NotNull T colorIfAbsent(final @Nullable TextColor color);

   default @NotNull T decorate(final @NotNull TextDecoration decoration) {
      return this.decoration(decoration, TextDecoration.State.TRUE);
   }

   default @NotNull T decorate(final @NotNull TextDecoration... decorations) {
      Map<TextDecoration, TextDecoration.State> map = new EnumMap(TextDecoration.class);
      int i = 0;

      for(int length = decorations.length; i < length; ++i) {
         map.put(decorations[i], TextDecoration.State.TRUE);
      }

      return this.decorations(map);
   }

   default @NotNull T decoration(final @NotNull TextDecoration decoration, final boolean flag) {
      return this.decoration(decoration, TextDecoration.State.byBoolean(flag));
   }

   @NotNull T decoration(final @NotNull TextDecoration decoration, final TextDecoration.@NotNull State state);

   @NotNull T decorationIfAbsent(final @NotNull TextDecoration decoration, final TextDecoration.@NotNull State state);

   @NotNull T decorations(final @NotNull Map<TextDecoration, TextDecoration.State> decorations);

   default @NotNull T decorations(final @NotNull Set<TextDecoration> decorations, final boolean flag) {
      return this.decorations((Map)decorations.stream().collect(Collectors.toMap(Function.identity(), (decoration) -> {
         return TextDecoration.State.byBoolean(flag);
      })));
   }

   @NotNull T clickEvent(final @Nullable ClickEvent event);

   @NotNull T hoverEvent(final @Nullable HoverEventSource<?> source);

   @NotNull T insertion(final @Nullable String insertion);
}
